; =====================================================
; Wohnraum-Uhr mit AT90S4434, AT90S8535 oder ATmega8535
; =====================================================
;
; V1.15, letzte Bearbeitung am 14.10.2007
;	 - Datumszhlung ergnzt: Beim Tageswechsel wird jetzt sofort
;	   das neue Datum angezeigt und nicht mehr auf eine DCF/HBG-
;	   Synchronisierung gewartet
;
; V1.14, letzte Bearbeitung am 23.09.2007
;	 - Der Anzeigemodus (Normalanzeige, Sekundenanzeige oder Tempe-
;	   raturanzeige) wird im EEPROM gespeichert, so dass die letzte
;	   Anzeige nach einem Stromausfall wieder hergestellt wird (fr
;	   Leute, die die Uhr als Temperatur-Groanzeige nutzen)
;
; V1.13, letzte Bearbeitung am 04.03.2007
;	 - Verbesserung der DCF/HBG-Dekodierung durch zustzlichen
;	   Zwischenspeicher, jetzt mssen 3 (anstatt 2 wie bisher)
;	   Minutenzyklen genau stimmen, damit die Uhr synchronisiert
;	   wird
;
; V1.12, letzte Bearbeitung am 18.02.2006
;	 - Die Uhr kann jetzt mit einem blinkenden Doppelpunkt ausge-
;	   stattet werden. Dafr ist allerdings eine nderung in der
;	   Schaltung erforderlich: an PortB7 (Pin 8) wird der Doppel-
;	   punkt angeschlossen und der Dezimalpunkt von LED 2 wird
;	   parallel zum Dezimalpunkt von LED 4 geschaltet - PortD7
;	   (Pin 21)
;
; V1.11, letzte Bearbeitung am 04.05.2005
;	 - Anpassung an das HBG75 Signal (Schweizer Zeitzeichensender),
;	   die Uhr kann jetzt wahlweise mit DCF77 oder HBG75 Empfnger
;	   betrieben werden
;
; V1.10, letzte Bearbeitung am 12.03.2005
;	 - Manuelle Einstellung der Zeit durch langes Drcken des
;	   Tasters (mind. 5 Sekunden) mglich, dabei kann nur die Zeit
;	   eingestellt werden
;	 - Anzeige von Temperaturwerten von -55 bis +60 Grad, Werte ab
;	   40 Grad werden ohne Nachkommastelle angezeigt
;	 - Normal- und Inversbetrieb der Ausgnge ist jetzt in einer
;	   Software vereint, der Inversbetrieb wird durch setzen des
;	   Adresse invfl1 auf 0xff eingeschaltet
;
; V1.02, letzte Bearbeitung am 06.03.2005
;	 - Verbesserung der DCF77-Auswertung
;
; V1.01, letzte Bearbeitung am 06.04.2003
;
;	----------------------
;	Belegung der I/O-Ports
;	----------------------
;
; A0: Ausgang LED 1 - Segment B (0=ein)
; A1: Ausgang LED 1 - Segment C (0=ein)
; A2: Ausgang LED 1 - Segment E (0=ein)
; A3: Ausgang LED 1 - Segment A,D,G (0=ein)
; A4: Eingang Bedientaster (mit Pullup)
; A5: Eingang DCF77-Empfnger (mit Pullup)
; A6: Ausgang Nachtschaltung (1=ein)
; A7: Eingang Nachtschaltung (analog)
;
; B0: Ausgang LED 2 - Segment A (0=ein)
; B1: Ausgang LED 2 - Segment B (0=ein)
; B2: Ausgang LED 2 - Segment C (0=ein)
; B3: Eingang LED 2 - Segment D (0=ein)
; B4: Eingang LED 2 - Segment E (0=ein)
; B5: Eingang LED 2 - Segment F (0=ein)
; B6: Eingang LED 2 - Segment G (0=ein)
; B7: Eingang LED 2 - Segment P (0=ein) -> alternativ Doppelpunkt
;
; C0: Ausgang LED 3 - Segment A (0=ein)
; C1: Ausgang LED 3 - Segment B (0=ein)
; C2: Ausgang LED 3 - Segment C (0=ein)
; C3: Ausgang LED 3 - Segment D (0=ein)
; C4: Ausgang LED 3 - Segment E (0=ein)
; C5: Ausgang LED 3 - Segment F (0=ein)
; C6: Ausgang LED 3 - Segment G (0=ein)
; C7: Ausgang Temperatursensor DS1620-RST (1=ein)
;
; D0: Ausgang LED 4 - Segment A + DS1620-DQ (0=ein)
; D1: Ausgang LED 4 - Segment B + DS1620-CLK (0=ein)
; D2: Ausgang LED 4 - Segment C (0=ein)
; D3: Ausgang LED 4 - Segment D (0=ein)
; D4: Ausgang LED 4 - Segment E (0=ein)
; D5: Ausgang LED 4 - Segment F (0=ein)
; D6: Ausgang LED 4 - Segment G (0=ein)
; D7: Ausgang LED 4 - Segment P (0=ein) -> alternativ LED 2, Segment P
;
;	------------------
;	Belegung der Timer
;	------------------
;
; Timer1: Interrupt alle 10ms (Vorteiler 1, Compare bei 39999)
;
;	-----------------------------------
;	Standard-Definitionen fr AT90S4434
;	-----------------------------------
;
.nolist
.include "4434def.inc"
.list
;
;	------------------------
;	Speicheradressen im SRAM
;	------------------------
;
.dseg
.org	0x60
;
pcount:	.byte	1	;Vorteiler (0-99) zhlt 10ms-Interrupts
second:	.byte	1	;Sekunde   (0-59)
minute:	.byte	1	;Minute    (0-59)
hour:	.byte	1	;Stunde    (0-23)
day:	.byte	1	;Tag       (1-31)
month:	.byte	1	;Monat     (1-12)
year:	.byte	1	;Jahr      (0-99)
dcftab:	.byte	6	;DCF77-Tabelle, eingehende Bits werden in
			;Byte 5, Bit 7 eingeschoben und alle weiteren
			;Bits um eine Position weitergerckt, Byte/Bit:
			;0/6-7: Zeitzone (6=1 Sommer, 7=1 Winter)
			;1/2-5: Minute Einer
			;1/6-7: Minute Zehner LSB
			;2/0:   Minute Zehner MSB
			;2/1:   Prfbit 1 (Minuten)
			;2/2-5: Stunde Einer
			;2/6-7: Stunde Zehner
			;3/0:   Prfbit 2 (Stunden)
			;3/1-4: Kalendertag Einer
			;3/5-6: Kalendertag Zehner
			;3/7:   Wochentag LSB
			;4/0-1: Wochentag MSB
			;4/2-5: Monat Einer
			;4/6:   Monat Zehner
			;4/7:   Jahr Einer LSB
			;5/0-2: Jahr Einer MSB
			;5/3-6: Jahr Zehner
			;5/7:   Prfbit 3 (Datum)
dcbuf1:	.byte	12	;DCF-Puffer 1, aktuelle decodierte Uhrzeit
			;Byte  0: Zeitzone
			;Byte  1: Minute Einer
			;Byte  2: Minute Zehner
			;Byte  3: Stunde Einer
			;Byte  4: Stunde Zehner
			;Byte  5: Kalendertag Einer
			;Byte  6: Kalendertag Zehner
			;Byte  7: Wochentag
			;Byte  8: Monat Einer
			;Byte  9: Monat Zehner
			;Byte 10: Jahr Einer
			;Byte 11: Jahr Zehner
dcbuf2:	.byte	12	;DCF-Puffer 2, vorherige decodierte Uhrzeit
			;+1 Minute, Aufbau wie dcbuf1
dcbuf3:	.byte	12	;DCF-Puffer 3, vorherige decodierte Uhrzeit
			;+2 Minuten, Aufbau wie dcbuf1
dimode:	.byte	1	;Anzeigemodus:
			;0: normale Zeitanzeige
			;1: Sekundenanzeige
			;2: Temperatur-Anzeige
			;3: Einstellung-Anzeige
			;4: Versionsnummer-Anzeige
			;5: Stunden-Anzeige und Eingabe
			;6: Minuten-Anzeige und Eingabe
			;7: Zeitanzeige, warten auf Tastendruck zur
			;   bernahme der neuen Zeit
ldimod:	.byte	1	;letzter Anzeigemodus (wird aus EEPROM gelesen)
			;0: normale Zeitanzeige
			;1: Sekundenanzeige
			;2: Temperatur-Anzeige
ditime:	.byte	1	;Zhler fr Anzeige-Timeout, zhlt rckwrts
			;auf 0 im Sekundentakt
diprog:	.byte	1	;Anzeige-Programm (wird in EERPOM gesichert),
			;wird auf Anzeige als 1,2,3 dargestellt:
			;0: Anzeige Zeit-Datum-Temperatur
			;1: Anzeige Zeit-Temperatur
			;2: Anzeige Zeit
nightf:	.byte	1	;Nacht-Flag (1=Nachtmodus), wird durch Haupt-
			;programm gesetzt und durch Interrupt-Routine
			;ausgewertet
adcval:	.byte	1	;letzter A/D-Wandler-Wert, wird von Interrupt-
			;Routine alle 10ms aktualisiert (reduziert auf
			;von 10 auf 8 Bit)
tempxa:	.byte	2	;Zwischenspeicher fr Temperaturberechnung
			;Wert des DS1620 Temperature-Registers
tempxb:	.byte	2	;Zwischenspeicher fr Temperaturberechnung
			;Wert des DS1620 Counter-Registers
tempxc:	.byte	2	;Zwischenspeicher fr Temperaturberechnung
			;Wert des DS1620 Slope-Registers
tempxd:	.byte	1	;Zwischenspeicher fr Vorzeichen
tempxe:	.byte	1	;Zwischenspeicher fr Dezimalpunkt-Abschaltung
tmpbcd:	.byte	3	;aktuelle Temperatur im BCD-Format (0.0-39.9)
			;Byte 0: Zehntel
			;Byte 1: Einer
			;Byte 2: Zehner
tmpflg:	.byte	1	;Temperatur-Flag
			;Bit 0: Temperaturwert aktuell (wird durch
			;       Hauptprogramm gesetzt und durch Inter-
			;       rupt-Routine alle 10s gelscht
			;Bit 1: neue Temperaturmessung erforderlich
			;       (wird von Interrupt-Routine im Nacht-
			;	modus stndlich gesetzt und vom Haupt-
			;	programm zurckgesetzt)
keytim:	.byte	1	;Zhler fr die Dauer der Tasterbettigung,
			;wird im Sekundentakt hochgezhlt
newflg:	.byte	1	;Neueingabe-Flag, wird gesetzt, wenn ein neuer
			;Stunden- oder Minutenwert eingegeben wurde
newmin:	.byte	1	;neuer Minutenwert von Tastereingabe
newhou:	.byte	1	;neuer Stundenwert von Tastereingabe
;
;	--------------------
;	Konstanten festlegen
;	--------------------
;
.equ	tm1end=	39999	;Timer1 Endwert (entspricht 100 Interrupts/s
			;oder Zykluszeit 10ms)
.equ	nlevel=	128	;Triggelpegel fr Nachtschaltung, entspricht
			;2,5V Schaltspannung
;
;	---------------------
;	Register-Definitionen
;	---------------------
;
.def	invflg=	r6	;Invers-Flag fr die LED-Ausgnge,
			;0= Normalbetrieb, 0xff= Inversbetrieb
.def	spflag=	r7	;Spezialflag fr die Doppelpunkt-Anzeige
			;0= Normalbetrieb, 0xff= Doppelpunktmodus
.def	dcfimp=	r10	;DCF77-Zhler, zhlt die empfangenen Impulse
			;innerhalb einer Minute (0-59)
.def	dcfcnt=	r11	;DCF77-Zhler, Impuls-Zhlung (Zyklen 10ms)
.def	keycnt=	r12	;Tasten-Zhler, zhlt aufeinanderfolgende
			;10ms-Interrupts bei gedrckter Taste
.def	flags=	r13	;Verschiedene Flags
			;Bit 0: akueller DCF77-Zustand
			;Bit 1: vorheriger DCF77-Zustand
			;Bit 2: Uhr wurde mind. einmal synchronisiert
			;Bit 3: Beginn einer neuen Minute (wird durch
			;	Timer gesetzt und vom Hauptprogramm
			;	zurckgesetzt)
			;Bit 4: Datum gltig
			;Bit 5: Temperatursensor vorhanden
			;Bit 6: Tastenstatus
			;	0= Taste 40ms losgelassen \ setzt Int-
			;	1= Taste 40ms gedrckt    / Routine
			;Bit 7: Tastenbearbeitung, wird von Haupt-
			;	Programm gesetzt und von Interrupt-
			;	Routine gelscht
.def	inttmp=	r14	;temporres Register fr Interrupt-Routinen
.def	intreg=	r15	;Zwischenspeicher fr SREG whrend der
			;Interrupt-Routinen
;
;	---------------------------------
;	Prgrammbeginn, Interrupt-Vektoren
;	---------------------------------
;
.cseg
.org	0x0000
;
reset:	rjmp	start			;Programmstart
exint0:	reti				;nicht genutzt
exint1:	reti				;nicht genutzt
t2comp:	reti				;nicht genutzt
t2ovfl:	reti				;nicht genutzt
t1icap:	reti				;nicht genutzt
t1coma:	rjmp	t1int			;Zeitzhlung alle 10ms
t1comb:	reti				;nicht genutzt
t1ovfl:	reti				;nicht genutzt
t0ovfl:	reti				;nicht genutzt
spiint:	reti				;nicht genutzt
uartrx:	reti				;nicht genutzt
uartem:	reti				;nicht genutzt
uarttx:	reti				;nicht genutzt
adccom:	reti				;nicht genutzt
eeprom:	reti				;nicht genutzt
ancomp:	reti				;nicht genutzt
;
;	- Initialisierung von Watchdog und RAM
;
start:	cli				;Interrupts sperren
	wdr				;Watchdog zurcksetzen
	ldi	r16,1<<wde		;Watchdog aktivieren, 15ms
	out	wdtcr,r16		;ausgeben
	ldi	r16,low(ramend)		;Stackpointer setzen
	out	spl,r16
	ldi	r16,high(ramend)
	out	sph,r16
	ldi	yl,0x60			;Beginn des RAM-Bereiches Low
	ldi	yh,0			;Beginn des RAM-Bereiches High
	clr	r16
init10:	st	y+,r16			;Speicherzelle lschen
	cpi	yl,low(ramend+1)	;RAM-Ende (Low) erreicht?
	brne	init10			;nein -> Schleife
	cpi	yh,high(ramend+1)	;RAM-Ende (High) erreicht?
	brne	init10			;nein -> Schleife
	clr	dcfimp			;DCF77-Impulszhler lschen
	clr	dcfcnt			;DCF77-Zhler lschen
	clr	keycnt			;Tastenzhler lschen
	clr	flags			;Flags initialisieren
;
;	- Flag fr Inversbetrieb lesen
;
	ldi	zl,low(2*invfl1)
	ldi	zh,high(2*invfl1)	;Zeiger auf Invers-Flag
	lpm				;Flag lesen
	mov	invflg,r0		;und in Flag-Register kopieren
;
;	- Flag fr den Doppelpunktmodus lesen
;
	ldi	zl,low(2*spflg1)
	ldi	zh,high(2*spflg1)	;Zeiger auf Spezial-Flag
	lpm				;Flag lesen
	mov	spflag,r0		;und in Flag-Register kopieren
;
;	- I/O Ports initialisieren
;
	ldi	r16,0b01001111		;PortA 4,5,7 als Eingang
	out	ddra,r16		;ausgeben
	ldi	r16,0b00111111		;PortA 6,7 mit Pullup, LED aus
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;ausgeben
	ser	r16			;PortB alles Ausgnge
	out	ddrb,r16		;ausgeben
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;PortB LED aus
	ser	r16			;PortB alles Ausgnge
	out	ddrc,r16		;PortC alles Ausgnge
	ldi	r16,0b01111111		;PortC DS1620-RST=0, LED aus
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;ausgeben
	ser	r16
	out	ddrd,r16		;PortD alles Ausgnge
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;PortD alle LEDs aus
;
	ldi	r18,20			;ca. 1s warten
init20:	rcall	wait50			;Warteschleife ca. 50 ms
	dec	r18
	brne	init20
;
	ldi	r16,high(tm1end)	;Timer1 Endwert1 H
	out	ocr1ah,r16		;ausgeben
	ldi	r16,low(tm1end)		;Timer1 Endwert1 L
	out	ocr1al,r16		;ausgeben
	clr	r16			;Timer1 kein PWM, kein Comp-Out
	out	tccr1a,r16
	ldi	r16,(1<<ctc1)+(1<<cs10)	;Timer1 Lschen nach Vergleich
	out	tccr1b,r16		;und Vorteiler=1 setzen
;
	ldi	r16,7			;PortA7 als Eingang fr den
	out	admux,r16		;A/D-Konverter whlen
	ldi	r16,(1<<aden)+(1<<adsc)+(1<<adfr)+(1<<adps2)+(1<<adps1)
	out	adcsr,r16		;A/D ein, Freerun, Vorteiler=64
;
;	- Daten aus EEPROM holen
;
	ldi	r16,low(edprog)		;Adresse des gespeicherten
	ldi	r17,high(edprog)	;Anzeigeprogramms
	rcall	eeread			;EERPOM lesen
	mov	r16,r0			;gelesenes Byte holen
	cpi	r16,3			;Wert>2?
	brcs	init30			;nein -> berspringen
	clr	r16			;sonst auf 0 setzen
init30:	sts	diprog,r16		;und speichern
;
	ldi	r16,low(edimod)		;Adresse des gespeicherten
	ldi	r17,high(edimod)	;Anzeigemodus
	rcall	eeread			;EERPOM lesen
	mov	r16,r0			;gelesenes Byte holen
	cpi	r16,3			;Wert>2?
	brcs	init40			;nein -> berspringen
	clr	r16			;sonst auf 0 setzen
init40:	sts	ldimod,r16		;und speichern
;
;	- Temperatursensor (DS1620) initialisieren
;
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xac		;Kommando "Read Config"
	rcall	w3send			;Kommando senden
	rcall	w3rec8			;8 Datenbits empfangen
	rcall	w3deac			;3Wire-Interface deaktivieren
	mov	r21,r22			;gelesenes Config-Byte kopieren
	andi	r21,0b00001100		;feste Bits filtern (Bit 3-2)
	cpi	r21,0b00001000		;feste Bits ok? (Bit 3-2=10)
	brne	init60			;nein -> kein Tempsensor vorh.
	set				;sonst Bit setzen fr
	bld	flags,5			;Temperatursensor vorhanden
	andi	r22,0x0f		;Bits 0-3 filtern
	cpi	r22,0x0b		;DS1620 richtig konfiguriert?
					;(CPU=1, 1Shot=1)
	breq	init50			;ja -> weiter, sonst
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0x0c		;Kommando "Write Config"
	rcall	w3send			;Kommando senden
	ldi	r22,0x03		;Config-Datenbyte (CPU=1,1Sh=1)
	rcall	w3send			;Datenbyte senden
	rcall	w3deac			;3Wire-Interface deaktivieren
	rcall	wait50			;50 ms warten
init50:	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xee		;Kommando "Start Conversion T"
	rcall	w3send			;Kommando senden
	rcall	w3deac			;3Wire-Interface deaktivieren
;
;	- Interrupt aktivieren
;
init60:	clr	r16
	out	gimsk,r16		;externe Interrupts sperren
	ldi	r16,1<<ocie1a
	out	timsk,r16		;Timer1 Vergleich1 freigeben
	sei
;
;	- Versionsnummer anzeigen
;
	ldi	r16,4			;Anzeigemodus fr Versions-
	sts	dimode,r16		;nummer setzen
	ldi	r16,3			;fr 3 Sekunden
	sts	ditime,r16		;Versionsnummer anzeigen
;
;
;	-------------------------------------------------------------
;	Hauptprogramm, DCF77 auswerten, Uhrzeit setzen, Tastenabfrage
;	und Auswertung, Anzeige entsprechend Mode
;	-------------------------------------------------------------
;
main:	bst	flags,3			;Beginn einer neuen Minute?
	brtc	ma0500			;nein -> weiter
	cli				;Interrupts sperren
	ldi	r16,0b11110111		;sonst Minuten-Flag lschen
	and	flags,r16		;in Flags speichern
	sei				;Interrupts wieder aktivieren
	rcall	mdcprc			;DCF77-Auswertung
	tst	r19			;DCF77 ok?
	brne	ma0500			;nein -> weiter
	rcall	mtsync			;sonst Uhr synchronisieren
;
ma0500:	lds	r16,adcval		;A/D-Wandler-Wert holen
	lds	r17,nightf		;Nachtflag holen
	tst	r17			;Nachtmodus aktiv?
	brne	ma0600			;ja -> bearbeiten
	cpi	r16,nlevel-5		;Nachtschwelle unterschritten?
	brcc	ma0700			;nein -> weiter
	ldi	r17,1			;sonst Nachtflag setzen
	rjmp	ma0700
ma0600:	cpi	r16,nlevel+5		;Tagschwelle berschritten?
	brcs	ma0700			;nein -> weiter
ma0650:	clr	r17			;sonst Nachtflag lschen
ma0700:	sts	nightf,r17		;Nachtflag wieder speichern
	tst	r17			;Nachtmodus aktiv?
	brne	ma0800			;ja -> bearbeiten
	cbi	porta,6			;sonst Anzeige auf Tag schalten
	rjmp	ma0900
ma0800:	sbi	porta,6			;Anzeige auf Nacht schalten
;
;	- Aktionen bei Ablauf des Anzeige- und Taster-Timeouts
;
ma0900:	lds	r19,dimode		;Anzeigemodus holen
	lds	r20,ditime		;Anzeigetimeout holen
	lds	r21,diprog		;Anzeigeprogramm holen
	lds	r16,keytim		;Tasterzhler holen
	cpi	r16,5			;Zhlerstand>=5?
	brcs	ma1000			;nein -> weiter
	clr	r16			;Zhlerstand lschen
	sts	keytim,r16		;und wieder speichern
	sts	newflg,r16		;Neueingabe-Flag lschen
	lds	r16,minute		;aktuellen Minutenwert holen
	sts	newmin,r16		;in Zwischenspeicher legen
	lds	r16,hour		;aktuellen Stundenwert holen
	sts	newhou,r16		;in Zwischenspeicher legen
	ldi	r19,5			;Anzeigemodus=5 setzen
	ldi	r20,5			;Anzeigetimeout=5s setzen
;
ma1000:	tst	r20			;Anzeigetimeout=0?
	brne	ma1100			;nein -> zur Tastenabfrage
	cpi	r19,4			;Anzeigemodus=4?
	breq	ma1030			;ja -> letzter Anzeigemodus
ma1010:	cpi	r19,3			;Anzeigemodus=3?
	brne	ma1040			;nein -> weiter
	ldi	r16,low(edprog)		;Adresse des gespeicherten
	ldi	r17,high(edprog)	;Anzeigeprogramms
	rcall	eeread			;EERPOM lesen
	cp	r0,r21			;Anzeigeprogramm-nderung?
	breq	ma1020			;nein -> nicht speichern
	mov	r18,r21			;sonst neuen Wert fr
	rcall	eewrit			;Anzeigeprogramm speichern
ma1020:	clr	r19			;Anzeigemodus=0 setzen
	rjmp	ma1100
;
ma1030:	lds	r19,ldimod		;letzten Anzeigemodus setzen
	rjmp	ma1100
;
ma1040:	cpi	r19,5			;Anzeigemodus=5?
	brne	ma1050			;nein -> weiter
	inc	r19			;sonst Anzeigemodus=6 setzen
	ldi	r20,5			;Timeout auf 5 Sekunden setzen
	rjmp	ma1100
;
ma1050:	cpi	r19,6			;Anzeigemodus=6?
	brne	ma1060			;nein -> weiter
	inc	r19			;sonst Anzeigemodus=7 setzen
	rjmp	ma1100
;
ma1060:	cpi	r19,7			;Anzeigemodus=7?
	brne	ma1100			;nein -> weiter
	lds	r16,newflg		;Neueingabe-Flag holen
	tst	r16			;wurden neue Daten eingegeben?
	brne	ma1100			;ja -> weiter
	clr	r19			;sonst Anzeigemodus=0
	clr	r20			;Anzeigetimeout=0
;
;	- Aktionen bei Erkennung eines Tastendrucks
;
ma1100:	bst	flags,6			;Taste gedrckt?
	brtc	ma1410			;nein -> Anzeigedaten speichern
	bst	flags,7			;Taste schon bearbeitet?
	brts	ma1410			;ja -> Anzeigedaten speichern
;
ma1120:	cli				;Interrupts sperren
	set
	bld	flags,7			;Bearbeitungsflag setzen
	sei				;Interrupts wieder freigeben
;
	cpi	r19,0			;Anzeigemodus=0? (Zeit)
	breq	ma1200			;ja -> bearbeiten
	cpi	r19,1			;Anzeigemodus=1? (Sekunden)
	breq	ma1300			;ja -> bearbeiten
	cpi	r19,2			;Anzeigemodus=2? (Temperatur)
	breq	ma1300			;ja -> bearbeiten
	cpi	r19,3			;Anzeigemodus=3? (Programm)
	brne	ma1400			;nein -> weiter
	ldi	r20,5			;Timeout auf 5 Sekunden setzen
	inc	r21			;Anzeigeprogramm erhhen
	cpi	r21,2			;Anzeigeprogramm=2?
	brne	ma1130			;nein -> weiter testen
	bst	flags,5			;Temperatursensor vorhanden?
	brts	ma1130			;ja -> weiter
	rjmp	ma1140			;sonst nur Programm 0/1 mglich
;
ma1130:	cpi	r21,3			;Anzeigeprogramm>2?
	brcs	ma1410			;nein -> Anzeigedaten speichern
ma1140:	clr	r21			;sonst Programm=0 setzen
	rjmp	ma2000			;und Anzeigedaten speichern
;
ma1200:	ldi	r20,5			;Timeout auf 5 Sekunden setzen
	inc	r19			;Anzeigemodus erhhen
	cpi	r19,2			;Anzeigemodus=2? (Temperatur)
	brne	ma1210			;nein -> weiter
	bst	flags,5			;Temperatursensor vorhanden?
	brts	ma1210			;ja -> weiter
	inc	r19			;sonst Temperatur berspringen
ma1210:	cpi	r19,4			;Anzeigemodus>4?
	brcc	ma1400			;ja -> weiter
	rjmp	ma2000			;sonst Anzeigedaten speichern
;
ma1300:	tst	r20			;Anzeigetimeout abgelaufen?
	brne	ma1200			;nein -> Anzeigemodus erhhen
ma1400:	cpi	r19,5			;Anzeigemodus>4
	brcc	ma1500			;ja -> weiter
	clr	r19			;sonst Anzeigemodus=0
	clr	r20			;Anzeigetimeout=0
ma1410:	rjmp	ma2000
;
ma1500:	cpi	r19,5			;Anzeigemodus=5? (Stundeneing.)
	brne	ma1600			;nein -> weiter
	lds	r16,newhou		;neuen Stundenwert holen
	inc	r16			;Wert erhhen
	cpi	r16,24			;Endwert 23 erreicht?
	brcs	ma1510			;nein -> weiter
	clr	r16			;sonst Wert zurcksetzen
ma1510:	sts	newhou,r16		;Stundenwert wieder speichern
	rjmp	ma1620
;
ma1600:	cpi	r19,6			;Anzeigemodus=6? (Minuteneing.)
	brne	ma1700			;nein -> weiter
	lds	r16,newmin		;neuen Minutenwert holen
	inc	r16			;Wert erhhen
	cpi	r16,60			;Endwert 60 erreicht?
	brcs	ma1610			;nein -> weiter
	clr	r16			;sonst Wert zurcksetzen
ma1610:	sts	newmin,r16		;Minutenwert wieder speichern
ma1620:	ldi	r16,1			;Wert fr Neueingabe-Flag
	sts	newflg,r16		;Neueingabe-Flag speichern
	ldi	r20,5			;Timeout auf 5 Sekunden setzen
	rjmp	ma2000
;
ma1700:	cpi	r19,7			;Anzeigemodus=7 (Zeitbernahme)
	brne	ma2000			;nein -> weiter
	cli				;sonst Interrupts sperren
	clr	r16
	sts	pcount,r16		;Vorteiler lschen
	sts	second,r16		;Sekunden lschen
	lds	r16,newmin		;neuen Minutenwert holen
	sts	minute,r16		;Minuten setzen
	lds	r16,newhou		;neuen Stundenwert holen
	sts	hour,r16		;Stunden setzen
	sei				;Interrupts wieder freigeben
	set
	bld	flags,2			;Synchronisations-Flag setzen
	clt
	bld	flags,4			;Datum ungltig
	clr	r19			;Anzeigemodus=0
	clr	r20			;Anzeigetimeout=0
;
ma2000:	sts	dimode,r19		;Anzeigemodus speichern
	sts	ditime,r20		;Anzeigetimeout speichern
	sts	diprog,r21		;Anzeigeprogramm speichern
	cpi	r19,3			;Anzeigemodus >2?
	brcc	ma2010			;ja -> weiter, sonst
	lds	r16,ldimod		;letzten Anzeigemodus holen
	cp	r16,r19			;wurde Anzeigemodus gendert?
	breq	ma2010			;nein -> weiter
	sts	ldimod,r19		;sonst neuen Modus speichern
	ldi	r16,low(edimod)		;EEPROM-Adresse letzter
	ldi	r17,high(edimod)	;Anzeigemodus
	mov	r18,r19			;Wert kopieren
	rcall	eewrit			;im EEPROM speichern
;
ma2010:	tst	r19			;Anzeigemodus=0? (Zeitanzeige)
	brne	ma3000			;nein -> weiter testen
	lds	r16,nightf		;Nachtflag holen
	tst	r16			;Nachtmodus aktiv?
	breq	ma2020			;nein -> weiter
	lds	r16,tmpflg		;Temperatur-Flags holen
	bst	r16,1			;neue Messung erforderlich?
	brtc	ma2060			;nein -> weiter zur Zeitanzeige
	clt				;Messung-Flag
	bld	r16,1			;lschen und
	sts	tmpflg,r16		;Flags wieder speichern
	cli				;sonst Interrupt sperren
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xee		;Kommando "Start Conversion T"
	rcall	w3send			;Kommando senden
	rcall	w3deac			;3Wire-Interface deaktivieren
	sei				;Interrupt wieder aktivieren
	rjmp	ma2060			;Zeit anzeigen
;
ma2020:	cpi	r21,2			;Anzeigeprogramm=2?
	breq	ma2060			;ja -> direkt zur Zeitanzeige
	lds	r16,second		;sonst Sekunden holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer holen
	cpi	r21,1			;Anzeigeprogramm=1?
	brne	ma2050			;nein -> weiter testen
	bst	flags,5			;Temperatursensor vorhanden?
	brtc	ma2060			;nein -> direkt zur Zeitanzeige
ma2030:	cpi	r18,7			;Sekunde<7?
	brcs	ma2060			;ja -> weiter mit Zeitanzeige
ma2040:	rcall	tmpout			;sonst Temperatur anzeigen
	rjmp	main
;
ma2050:	bst	flags,4			;Datum gltig?
	brts	ma2070			;ja -> weiter
	bst	flags,5			;Temperatursensor vorhanden?
	brts	ma2030			;ja -> Zeit/Temperatur anzeigen
ma2060:	rcall	clkout			;Zeit anzeigen
	rjmp	main
;
ma2070:	bst	flags,5			;Temperatursensor vorhanden?
	brts	ma2090			;ja -> weiter
	cpi	r18,7			;Sekunde<7?
	brcs	ma2060			;ja -> weiter mit Zeitanzeige
ma2080:	rcall	datout			;sonst Datum ausgeben
	rjmp	main
;
ma2090:	cpi	r18,6			;Sekunde<6?
	brcs	ma2060			;ja -> weiter mit Zeitanzeige
	cpi	r18,8			;Sekunde<8?
	brcs	ma2080			;ja -> Datum anzeigen
	rjmp	ma2040			;sonst Temperatur anzeigen
;
ma3000:	cpi	r19,1			;Anzeigemodus=1? (Sekunden)
	brne	ma3100			;nein -> weiter testen
	rcall	secout			;Sekunden anzeigen
	rjmp	main
;
ma3100:	cpi	r19,2			;Anzeigemodus=2? (Temperatur)
	brne	ma3200			;nein -> weiter testen
	rcall	tmpout			;Temperatur anzeigen
	rjmp	main
;
ma3200:	cpi	r19,3			;Anzeigemodus=3? (Programm)
	brne	ma3300			;nein -> weiter testen
	rcall	prgout			;Programm anzeigen
	rjmp	main
;
ma3300:	cpi	r19,4			;Anzeigemodus=4? (Versions-Nr.)
	brne	ma3400			;nein -> weiter testen
	rcall	verout			;Versionsnummer anzeigen
	rjmp	main
;
ma3400:	cpi	r19,5			;Anzeigemodus=5? (Stundeneing.)
	brne	ma3500			;nein -> weiter testen
	rcall	houout			;Stundeneingabe anzeigen
	rjmp	main
;
ma3500:	cpi	r19,6			;Anzeigemodus=6? (Minuteneing.)
	brne	ma3600			;nein -> weiter testen
	rcall	minout			;Minuteneingabe anzeigen
	rjmp	main
;
ma3600:	cpi	r19,7			;Anzeigemodus=7? (Warten)
	brne	ma3700			;nein -> weiter testen
	rcall	setout			;eingegebene Zeit anzeigen
ma3700:	rjmp	main
;
;	---------------------------------------------
;	DCF77-Auswertung, Kontrolle der Prfbits 1-3,
;	Ablegen der Zeitinformation und Vergleich mit
;	vorheriger Zeitinformation
;	Register: r18,r19,r20,r21,r22,r23,y,z
;	Ausgang: r19=0 - alles ok, r19>0 - Fehler
;	---------------------------------------------
;
;	- Kontrolle des Impulszhlers und der Prfbits fr Minuten,
;	  Stunden und Datum
;
mdcprc:	clr	r19			;Fehlerzhler lschen
	mov	r21,dcfimp		;DCF77-Impulszhler kopieren
	clr	dcfimp			;DCF77-Impulszhler lschen
	cpi	r21,59			;wurden 59 Impulse empfangen?
	breq	mdc010			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
	rjmp	mdc200			;Auswertung vorzeitig beenden
;
mdc010:	clr	r21			;wird fr Parittsber. bentigt
	clr	r22			;Zhler fr Parittsberechnung
	lds	r23,dcftab+1		;Byte 1 aus Tabelle holen
	ldi	r20,6			;6 Minuten-Bits auswerten
mdc020:	rol	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	dec	r20			;alle Bits ausgewertet?
	brne	mdc020			;nein -> Schleife
	lds	r23,dcftab+2		;Byte 2 aus Tabelle holen
	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	bst	r22,0			;war das Ergebnis geradzahlig?
	brtc	mdc030			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
;
mdc030:	clr	r22			;Zhler fr Parittsberechnung
	ldi	r20,6			;6 Stunden-Bits auswerten
mdc040:	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	dec	r20			;alle Bits ausgewertet?
	brne	mdc040			;nein -> Schleife
	lds	r23,dcftab+3		;Byte 3 aus Tabelle holen
	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	bst	r22,0			;war das Ergebnis geradzahlig?
	brtc	mdc050			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
;
mdc050:	clr	r22			;Zhler fr Parittsberechnung
	ldi	r20,7			;7 Datum-Bits auswerten
mdc060:	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	dec	r20			;alle Bits ausgewertet?
	brne	mdc060			;nein -> Schleife
	lds	r23,dcftab+4		;Byte 4 aus Tabelle holen
	ldi	r20,8			;8 Datum-Bits auswerten
mdc070:	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	dec	r20			;alle Bits ausgewertet?
	brne	mdc070			;nein -> Schleife
	lds	r23,dcftab+5		;Byte 5 aus Tabelle holen
	ldi	r20,8			;8 Datum-Bits auswerten
mdc080:	ror	r23			;relevantes Bit in Carry schieb
	adc	r22,r21			;Eins-Bit addieren
	dec	r20			;alle Bits ausgewertet?
	brne	mdc080			;nein -> Schleife
	bst	r22,0			;war das Ergebnis geradzahlig?
	brtc	mdc090			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
mdc090:	tst	r19			;Parittsfehler aufgetreten?
	breq	mdc100			;nein -> weiter
	rjmp	mdc200			;sonst DCF-Tabelle lschen
;
;	- Ablegen der Zeitinformation in Puffer 1
;
mdc100:	ldi	zl,low(dcbuf1)		;Zeiger auf DCF77-Puffer 1
	ldi	zh,high(dcbuf1)
	lds	r23,dcftab		;Byte 0 aus Tabelle holen
	rol	r23			;Bit 6 auf Bit 0 schieben
	rol	r23
	rol	r23
	andi	r23,0x01		;relevantes Bit filtern
	st	z+,r23			;Zeitzone speichern
	lds	r23,dcftab+1		;Byte 1 aus Tabelle holen
	ror	r23			;Bits 2-5 auf Bits 0-3 schieben
	ror	r23
	andi	r23,0x0f		;relevante Bits filtern
	st	z+,r23			;Minuten Einer speichern
	lds	r23,dcftab+1		;Byte 1 aus Tabelle holen
	lds	r22,dcftab+2		;Byte 2 aus Tabelle holen
	rol	r23			;Bits 6-7 aus r23 und 0 aus r22
	rol	r22			;auf Bits 0-2 von r22 schieben
	rol	r23
	rol	r22
	andi	r22,0x07		;relevante Bits filtern
	st	z+,r22			;Minuten Zehner speichern
	lds	r23,dcftab+2		;Byte 2 aus Tabelle holen
	ror	r23			;Bits 2-5 auf Bits 0-3 schieben
	ror	r23
	andi	r23,0x0f		;relevante Bits filtern
	st	z+,r23			;Stunden Einer speichern
	lds	r23,dcftab+2		;Byte 2 aus Tabelle holen
	rol	r23			;Bits 6-7 auf Bits 0-1 schieben
	rol	r23
	rol	r23
	andi	r23,0x03		;relevante Bits filtern
	st	z+,r23			;Stunden Zehner speichern
	lds	r23,dcftab+3		;Byte 3 aus Tabelle holen
	ror	r23			;Bits 1-4 auf Bits 0-3 schieben
	andi	r23,0x0f		;relevante Bits filtern
	st	z+,r23			;Kalendertag Einer speichern
	lds	r23,dcftab+3		;Byte 3 aus Tabelle holen
	ror	r23			;Bits 5-6 auf Bits 4-5 schieben
	swap	r23			;Nibbles tauschen
	andi	r23,0x03		;relevante Bits filtern
	st	z+,r23			;Kalendertag Zehner speichern
	lds	r23,dcftab+3		;Byte 3 aus Tabelle holen
	rol	r23			;Bit 7 in Carry schieben
	lds	r23,dcftab+4		;Byte 4 aus Tabelle holen
	rol	r23			;Carry in Bit 0 schieben
	andi	r23,0x07		;relevante Bits filtern
	st	z+,r23			;Wochentag speichern
	lds	r23,dcftab+4		;Byte 4 aus Tabelle holen
	ror	r23			;Bits 2-5 in Bits 0-3 schieben
	ror	r23
	andi	r23,0x0f		;relevante Bits filtern
	st	z+,r23			;Monat Einer speichern
	lds	r23,dcftab+4		;Byte 4 aus Tabelle holen
	rol	r23			;Bit 6 in Bit 0 schieben
	rol	r23
	rol	r23
	andi	r23,0x01		;relevantes Bit filtern
	st	z+,r23			;Monat Zehner speichern
	lds	r23,dcftab+4		;Byte 4 aus Tabelle holen
	rol	r23			;Bit 7 in Carry schieben
	lds	r23,dcftab+5		;Byte 5 aus Tabelle holen
	rol	r23			;Carry in Bit 0 schieben
	andi	r23,0x0f		;relevante Bits filtern
	st	z+,r23			;Jahr Einer speichern
	lds	r23,dcftab+5		;Byte 5 aus Tabelle holen
	rol	r23			;Bits 3-6 in Bits 4-7 schieben
	swap	r23			;Nibbles tauschen
	andi	r23,0x0f		;relevante Bits filtern
	st	z+,r23			;Jahr Zehner speichern
;
;	- Vergleich des aktuellen Puffers mit der vorherigen Zeitinfor-
;	  mation, gleichzeitig Puffer 2->3 und Puffer 1-> 2 kopieren
;
	ldi	yl,low(dcbuf1)
	ldi	yh,high(dcbuf1)		;Zeiger auf DCF77-Puffer 1
	ldi	zl,low(dcbuf2)
	ldi	zh,high(dcbuf2)		;Zeiger auf DCF77-Puffer 2
	ldi	xl,low(dcbuf3)
	ldi	xh,high(dcbuf3)		;Zeiger auf DCF77-Puffer 3
	clr	r18			;Nullbyte-Detektor lschen
	ldi	r23,12			;12 Bytes vergleichen
mdc110:	ld	r22,y+			;Puffer 1 Inhalt holen
	or	r18,r22			;Nullbyte-Detektor bearbeiten
	ld	r21,z			;Puffer 2 Inhalt holen
	ld	r20,x			;Puffer 3 Inhalt holen
	st	x+,r21			;Puffer 2 nach 3 kopieren
	st	z+,r22			;Puffer 1 nach 2 kopieren
	cp	r21,r22			;Inhalt P1/P2 identisch?
	brne	mdc120			;nein -> Fehler
	cp	r20,r21			;Inhalt P2/P3 identisch?
	breq	mdc130			;ja -> weiter
mdc120:	inc	r19			;sonst Fehlerzhler erhhen
mdc130:	dec	r23			;alle Bytes bearbeitet?
	brne	mdc110			;nein -> Schleife
	tst	r18			;alles Nullbytes im Puffer?
	brne	mdc140			;nein -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
;
;	- Puffer 2 und 3 um eine Minute erhhen, es werden nur Minuten
;	  und Stunden bercksichtigt
;
mdc140:	ldi	zl,low(dcbuf2)
	ldi	zh,high(dcbuf2)		;Zeiger auf DCF77-Puffer 2
	rcall	incdcb			;um eine Minute erhhen
	ldi	zl,low(dcbuf3)
	ldi	zh,high(dcbuf3)		;Zeiger auf DCF77-Puffer 3
	rcall	incdcb			;um eine Minute erhhen
;
;	- Ende, DCF-Tabelle lschen
;
mdc200:	ldi	zl,low(dcftab)		;Zeiger auf DCF77-Tabelle
	ldi	zh,high(dcftab)
	ldi	r23,6			;6 Bytes bearbeiten
	clr	r22			;Nullwert
mdc210:	st	z+,r22			;Tabellenplatz lschen
	dec	r23			;alle Pltze bearbeitet?
	brne	mdc210			;nein -> Schleife
	ret
;
;	------------------------------------------------------
;	Zeitinformationen in Uhrenbereich kopieren und Uhrzeit
;	synchronisieren
;	------------------------------------------------------
;
mtsync:	cli				;Interrupts sperren
	lds	r0,dcbuf1+1		;Minuten Einer holen
	lds	r1,dcbuf1+2		;Minuten Zehner holen
	rcall	bcdhex			;in Hexcode wandeln
	sts	minute,r16		;und speichern
	lds	r0,dcbuf1+3		;Stunden Einer holen
	lds	r1,dcbuf1+4		;Stunden Zehner holen
	rcall	bcdhex			;in Hexcode wandeln
	sts	hour,r16		;und speichern
	lds	r0,dcbuf1+5		;Tag Einer holen
	lds	r1,dcbuf1+6		;Tag Zehner holen
	rcall	bcdhex			;in Hexcode wandeln
	sts	day,r16			;und speichern
	lds	r0,dcbuf1+8		;Monat Einer holen
	lds	r1,dcbuf1+9		;Monat Zehner holen
	rcall	bcdhex			;in Hexcode wandeln
	sts	month,r16		;und speichern
	lds	r0,dcbuf1+10		;Jahr Einer holen
	lds	r1,dcbuf1+11		;Jahr Zehner holen
	rcall	bcdhex			;in Hexcode wandeln
	sts	year,r16		;und speichern
	clr	r16
	sts	second,r16		;Sekundenzhler lschen
	sts	pcount,r16		;Vorteiler lschen
	ldi	r16,0b00010100		;Synchron- und Datum-Bit setzen
	or	flags,r16		;in Flags einsetzen
	sei				;Interrupts wieder aktivieren
	ret
;
;	---------------------------
;	Aktuelle Zeit ausgeben
;	Register: r0,r1,r16,r17,r18
;	---------------------------
;
clkout:	lds	r16,hour		;Stunden holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s2			;in 7-Segment-Code wandeln
	in	r17,porta		;aktuellen PortA Status holen
	andi	r17,0xf0		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
;
	bst	flags,2			;Uhr schon synchronisiert?
	brts	clko10			;ja -> Blinken
	bst	flags,0			;DCF77-Status holen
	bld	r16,7			;als Dezimalpunkt setzen
	ldi	r17,0x80
	eor	r16,r17			;invertieren
	rjmp	clko20			;und ausgeben
clko10:	lds	r17,pcount		;sonst Vorteiler holen
	cpi	r17,50			;Wert<50 (Punkt 2 ein)?
	brcs	clko20			;ja -> weiter
	ori	r16,0x80		;sonst Punkt 2 ausschalten
clko20:	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;ausgeben
;
	lds	r16,minute		;Minuten holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimal-Punkt 4 ausschalten
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16
	ret
;
;	----------------------------------
;	Sekunden und DCF77-Signal ausgeben
;	Register: r0,r1,r16,r17,r18,r19,z
;	----------------------------------
;
secout:	in	r16,porta		;aktuellen PortA Status holen
	ori	r16,0x0f		;alle Segmente auschalten
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	ser	r16			;PortB alle Segmente aus
	bst	flags,0			;DCF77-Status holen
	bld	r16,7			;als Dezimalpunkt setzen
	ldi	r17,0x80
	eor	r16,r17			;invertieren
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;und ausgeben
	lds	r16,second		;Sekunden holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r19,r16			;Zehner-Segmente sichern
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimalpunkt lschen
	lds	r17,ditime		;Anzeigetimout-Zhler holen
	tst	r17			;Zeit abgelaufen?
	breq	seco20			;ja -> Sekunden ausgeben
	lds	r17,pcount		;Vorteiler holen
	cpi	r17,50			;Wert<50?
	brcs	seco10			;ja -> weiter
	subi	r17,50			;sonst 50 subtrahieren
seco10:	cpi	r17,45			;Wert<45 (Anzeige ein)
	brcs	seco20			;ja -> Anzeige
	ori	r19,0x7f		;sonst alle Z-Segmente lschen
	ori	r16,0xff		;und alle E-Segmente lschen
seco20:	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r19,r17			;LED-Ports invertieren
	out	portc,r19		;Zehner ausgeben
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;Einer ausgeben
	ret
;
;	-----------------------------
;	Aktuelles Datum ausgeben
;	Register: r0,r1,r16,r17,r18,z
;	-----------------------------
;
datout:	lds	r16,day			;Tag holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s2			;in 7-Segment-Code wandeln
	in	r17,porta		;aktuellen PortA Status holen
	andi	r17,0xf0		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	tst	spflag			;Doppelpunktmodus aktiv?
	breq	dato10			;nein -> weiter
	ori	r16,0x80		;sonst Dezimalpunkt ausschalten
dato10:	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;ausgeben
;
	lds	r16,month		;Monat holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	tst	r16			;Zehner=0?
	brne	dato20			;nein -> weiter
	ldi	r16,15			;sonst durch Leerzei. ersetzen
dato20:	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;ausgeben
	ret
;
;	---------------------------------------------------
;	Aktuelle Temperatur ausgeben
;	Register: r0,r1,r8,r9,r16,r17,r18,r20,r21,r22,r23,z
;	---------------------------------------------------
;
tmpout:	lds	r16,tmpflg		;Temperatur-Flags holen
	bst	r16,0			;Temperatur gltig?
	brtc	tmp010			;nein -> neu einlesen
	rjmp	tmp100			;sonst anzeigen
;
tmp010:	set				;sonst Temperatur-Flag
	bld	r16,0			;setzen
	sts	tmpflg,r16		;und Flags wieder speichern
	cli				;Interrupt sperren
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xaa		;Kommando "Read Temperature"
	rcall	w3send			;Kommando senden
	rcall	w3rec9			;9 Datenbits empfangen
	rcall	w3deac			;3Wire-Interface deaktivieren
	sts	tempxa,r22		;Temperatur-Wert L speichern
	sts	tempxa+1,r23		;Temperatur-Wert H speichern
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xa0		;Kommando "Read Counter"
	rcall	w3send			;Kommando senden
	rcall	w3rec9			;9 Datenbits empfangen
	rcall	w3deac			;3Wire-Interface deaktivieren
	sts	tempxb,r22		;Counter-Wert L speichern
	sts	tempxb+1,r23		;Counter-Wert H speichern
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xa9		;Kommando "Read Slope"
	rcall	w3send			;Kommando senden
	rcall	w3rec9			;9 Datenbits empfangen
	rcall	w3deac			;3Wire-Interface deaktivieren
	sts	tempxc,r22		;Slope-Wert L speichern
	sts	tempxc+1,r23		;Slope-Wert H speichern
	rcall	w3actv			;3Wire-Interface aktivieren
	ldi	r22,0xee		;Kommando "Start Conversion T"
	rcall	w3send			;Kommando senden
	rcall	w3deac			;3Wire-Interface deaktivieren
	sei				;Interrupt wieder aktivieren
;
	lds	r16,tempxa		;Temperatur-Wert L holen
	lds	r17,tempxa+1		;Temperatur-Wert H holen
	tst	r17			;Temperatur-Wert negativ?
	brne	tmp040			;ja -> bearbeiten
	cpi	r16,120			;Temperatur-Wert>=120 (60C)?
	brcs	tmp040			;nein -> bearbeiten
	ldi	r16,13			;sonst Temperatur
	sts	tmpbcd,r16		;auf ?.? setzen
	sts	tmpbcd+1,r16
	clr	r16
	sts	tmpbcd+2,r16		;H-Wert=0
	rjmp	tmp100			;direkt anzeigen
;
;	Berechnung der Temperatur nach Datenblatt DS1620 (AN-068),
;	Es wird mit 20-fachen Werten gerechnet, damit ganze Zahlen
;	verwendet werden knnen.
;
tmp040:	lds	r16,tempxc		;Slope-Wert holen
	lds	r17,tempxc+1
	lds	r18,tempxb		;Counter-Wert holen
	lds	r19,tempxb+1
	sub	r16,r18			;Subtraktion: Slope - Counter
	sbc	r17,r19
	ldi	r18,low(20)		;Multiplikation mit 20
	ldi	r19,high(20)
	rcall	mul16u
	mov	r16,r18			;Ergebnis nach r16,r17 als
	mov	r17,r19			;Divident
	lds	r18,tempxc		;Slope-Wert holen (Divisor)
	lds	r19,tempxc+1
	rcall	div16u			;Division
	mov	r8,r16			;Ergebnis in r8,r9 sichern
	mov	r9,r17
	lds	r18,tempxa		;Temperatur-Wert L holen
	asr	r18			;LSB abschneiden
	lds	r19,tempxa+1		;Temperatur-Wert H holen
	tst	r19			;Wert negativ?
	breq	tmp050			;nein -> berspringen
	ser	r19			;H=0xff setzen (Zweierkomplem.)
tmp050:	ldi	r16,low(100)		;Offset 100 zur Vermeidung von
	ldi	r17,high(100)		;negativen Zahlen addieren,
	add	r18,r16			;(wird spter wieder subtra-
	adc	r19,r17			;hiert)
	ldi	r16,low(20)		;Multiplikation mit 20
	ldi	r17,high(20)
	rcall	mul16u
	ldi	r20,low(5)		;Subtrahend 0,25*20
	ldi	r21,high(5)
	sub	r18,r20			;Subtraktion Temp - 0,25*20
	sbc	r19,r21
	add	r18,r8			;Addition Temp + Quotient aus
	adc	r19,r9			;erster Rechenoperation
	mov	r16,r18			;Ergebnis nach r16,r17 kopieren
	mov	r17,r19
	asr	r17			;Division durch 0, ergibt den
	ror	r16			;Temperaturwert in Zehntel Grad
	ldi	r18,low(1000)		;Offset 100.0 wieder abziehen
	ldi	r19,high(1000)
	sub	r16,r18
	sbc	r17,r19
	clr	r21			;Flag fr negative Temp lschen
	bst	r17,7			;Temperaturwert negativ?
	brtc	tmp060			;nein -> anzeigen
	clr	r18			;sonst Zweierkomplement berech-
	clr	r19			;nen, 0x10000 laden
	sub	r18,r16			;und Tempwert subtrahieren
	sbc	r19,r17
	mov	r16,r18			;Ergebnis nach r16,r17 kopieren
	mov	r17,r19
	ser	r21			;Flag fr negative Temp setzen
tmp060:	sts	tempxd,r21		;Flag fr Vorzeichen speichern
	rcall	binbcd			;Ergebnis in BCD wandeln
	sts	tmpbcd,r0		;und speichern
	sts	tmpbcd+1,r1
	sts	tmpbcd+2,r2
;
	clr	r18			;Flag: Dezimalpunkt einschalten
	lds	r16,tmpbcd+2		;Temperaturwert H holen
	cpi	r16,4			;H-Wert>3?
	brcs	tmp070			;nein -> weiter
	lds	r17,tmpbcd+1		;sonst Temperaturwert M holen
	sts	tmpbcd,r17		;als Temperaturwert L speichern
	sts	tmpbcd+1,r16		;H als Temp-Wert M speichern
	clr	r16
	sts	tmpbcd+2,r16		;H-Wert lschen
	ser	r18			;Flag: Dezimalpunkt ausschalten
tmp070:	sts	tempxe,r18		;Flag speichern
;
tmp100:	lds	r16,tmpbcd+2		;Temperaturwert H holen
	rcall	bcd7s2			;in 7-Segment-Code wandeln
	in	r17,porta		;aktuellen PortA Status holen
	andi	r17,0xf0		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	lds	r16,tmpbcd+1		;Temperaturwert M holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	tst	spflag			;Doppelpunktmodus aktiv?
	brne	tmp110			;ja -> Dezimalpunkt ausschalten
	lds	r18,tempxe		;Flag fr Dezimalpunkt holen
	tst	r18			;Dezimalpunkt verwenden?
	breq	tmp120			;ja -> weiter
tmp110:	ori	r16,0x80		;sonst Dezimalpunkt ausschalten
tmp120:	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;ausgeben
	lds	r16,tmpbcd		;Temperaturwert L holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	ldi	r16,12			;Zeichencode fr "" (Grad)
	lds	r17,tempxd		;Flag fr Vorzeichen holen
	tst	r17			;Temperaturwert negativ?
	breq	tmp130			;nein -> weiter
	ldi	r16,14			;sonst Zeichencode fr "-"
tmp130:	rcall	bcd7s1			;in 7-Segment-Code wandeln
	tst	spflag			;Doppelpunktmodus aktiv?
	breq	tmp140			;nein -> Dezmalpunkt ausschalt.
	lds	r18,tempxe		;Flag fr Dezimalpunkt holen
	tst	r18			;Dezimalpunkt verwenden?
	breq	tmp150			;ja -> weiter
tmp140:	ori	r16,0x80		;sonst Dezimalpunkt ausschalten
tmp150:	lds	r17,ditime		;Anzeigetimout-Zhler holen
	tst	r17			;Zeit abgelaufen?
	breq	tmp170			;ja -> Zeichen ausgeben
	lds	r17,pcount		;Vorteiler holen
	cpi	r17,50			;Wert<50?
	brcs	tmp160			;ja -> weiter
	subi	r17,50			;sonst 50 subtrahieren
tmp160:	cpi	r17,45			;Wert<45 (Anzeige ein)
	brcs	tmp170			;ja -> Anzeige
	ser	r16			;sonst alle Segmente lschen
tmp170:	eor	r16,invflg		;LED-POrts invertieren
	out	portd,r16		;und ausgeben
	ret
;
;	-----------------------------
;	Programm-Einstellung ausgeben
;	Register: r0,r1,r16,r17,z
;	-----------------------------
;
prgout:	in	r16,porta		;aktuellen PortA Status holen
	ori	r16,0x0f		;alle Segmente auschalten
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	ldi	r16,10			;Code fr Zeichen "P"
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimalpunkt lschen
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;und ausgeben
	ldi	r16,11			;Code fr Zeichen "r"
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	lds	r16,diprog		;Anzeigeprogramm holen
	inc	r16			;Anzeigekorrektur
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimalpunkt lschen
	lds	r17,pcount		;Vorteiler holen
	cpi	r17,50			;Wert<50?
	brcs	prgo10			;ja -> weiter
	subi	r17,50			;sonst 50 subtrahieren
prgo10:	cpi	r17,45			;Wert<45 (Anzeige ein)
	brcs	prgo20			;ja -> Anzeige
	ser	r16			;sonst alle Segmente lschen
prgo20:	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;und ausgeben
	ret
;
;	----------------------------------
;	Versionsnummer ausgeben
;	Register: r0,r1,r2,r3,r4,r16,r17,z
;	----------------------------------
;
verout:	in	r16,porta		;aktuellen PortA Status holen
	ori	r16,0x0f		;Segmente auschalten
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	ldi	zl,low(versnr*2+1)	;Zeiger auf gltigen Teil der
	ldi	zh,high(versnr*2+1)	;Versionsnummer
	lpm				;erste Ziffer holen
	mov	r2,r0			;und zwischenspeichern
	ld	r0,z+			;Zeiger auf nchste Ziffer
	lpm				;zweite Ziffer holen
	mov	r3,r0			;und zwischenspeichern
	ld	r0,z+			;Zeiger auf nchste Ziffer
	lpm				;dritte Ziffer holen
	mov	r4,r0			;und zwischenspeichern
	mov	r16,r2			;gesicherte erste Ziffer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;ausgeben
	mov	r16,r3			;gesicherte zweite Ziffer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	mov	r16,r4			;gesicherte zweite Ziffer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimalpunkt lschen
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;ausgeben
	ret
;
;	-----------------------------------
;	Neue Zeit anzeigen, Stunden blinken
;	Register: r0,r1,r16,r17,r18
;	-----------------------------------
;
houout:	lds	r16,newhou		;eingegebene Stunden holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s2			;in 7-Segment-Code wandeln
	in	r17,porta		;aktuellen PortA Status holen
	andi	r17,0xf0		;Segmente lschen
	or	r17,r16			;mit neuen Segmenten verknpfen
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
;
	lds	r18,pcount		;Vorteiler holen
	cpi	r18,50			;Wert<50?
	brcs	houo10			;ja -> weiter
	subi	r18,50			;sonst 50 subtrahieren
houo10:	cpi	r18,45			;Wert<45 (Anzeige ein)
	brcs	houo20			;ja -> Anzeige
	ori	r17,0x0f		;sonst alle Z-Segmente lschen
	ori	r16,0x7f		;und alle E-Segmente lschen
houo20:	mov	r18,invflg		;Invers-Flag holen
	andi	r18,0x0f		;LED-Ports filtern
	eor	r17,r18			;LED-Ports invertieren
	out	porta,r17		;Zehner ausgeben
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;Einer ausgeben
;
	lds	r16,newmin		;eingegebene Minuten holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimal-Punkt 4 ausschalten
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;ausgeben
	ret
;
;	-----------------------------------
;	Neue Zeit anzeigen, Minuten blinken
;	Register: r0,r1,r16,r17,r18
;	-----------------------------------
;
minout:	lds	r16,newhou		;eingegebene Stunden holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s2			;in 7-Segment-Code wandeln
	in	r17,porta		;aktuellen PortA Status holen
	andi	r17,0xf0		;Segmente lschen
	or	r17,r16			;mit neuen Segmenten verknpfen
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	mov	r18,invflg		;Invers-Flag holen
	andi	r18,0x0f		;LED-Ports filtern
	eor	r17,r18			;LED-Ports invertieren
	out	porta,r17		;Zehner ausgeben
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;Einer ausgeben
;
	lds	r16,newmin		;eingegebene Minuten holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r17,r16			;mit neuen Segmenten verknpfen
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimal-Punkt 4 ausschalten
;
	lds	r18,pcount		;Vorteiler holen
	cpi	r18,50			;Wert<50?
	brcs	mino10			;ja -> weiter
	subi	r18,50			;sonst 50 subtrahieren
mino10:	cpi	r18,45			;Wert<45 (Anzeige ein)
	brcs	mino20			;ja -> Anzeige
	ori	r17,0x7f		;sonst alle Z-Segmente lschen
	ori	r16,0x7f		;und alle E-Segmente lschen
mino20:	mov	r18,invflg		;Invers-Flag holen
	andi	r18,0x7f		;LED-Ports filtern
	eor	r17,r18			;LED-Ports invertieren
	out	portc,r17		;und ausgeben
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;ausgeben
	ret
;
;	------------------------------------
;	Neue Zeit anzeigen, warten auf Taste
;	Register: r0,r1,r16,r17,r18
;	------------------------------------
;
setout:	lds	r16,newhou		;eingegebene Stunden holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s2			;in 7-Segment-Code wandeln
	in	r17,porta		;aktuellen PortA Status holen
	andi	r17,0xf0		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x0f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	porta,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	eor	r16,invflg		;LED-Ports invertieren
	out	portb,r16		;ausgeben
;
	lds	r16,newmin		;eingegebene Minuten holen
	rcall	hexbcd			;in BCD-Zahlen wandeln
	mov	r18,r0			;Einer sichern
	mov	r16,r1			;Zehner holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	in	r17,portc		;aktuellen PortC Status holen
	andi	r17,0x80		;Segmente lschen
	or	r16,r17			;mit neuen Segmenten verknpfen
	mov	r17,invflg		;Invers-Flag holen
	andi	r17,0x7f		;LED-Ports filtern
	eor	r16,r17			;LED-Ports invertieren
	out	portc,r16		;und ausgeben
	mov	r16,r18			;gesicherte Einer holen
	rcall	bcd7s1			;in 7-Segment-Code wandeln
	ori	r16,0x80		;Dezimal-Punkt 4 ausschalten
	eor	r16,invflg		;LED-Ports invertieren
	out	portd,r16		;ausgeben
	ret
;
;	-------------------------
;	Warteschleife ca. 49,3 ms
;	Register: r16,r17
;	-------------------------
;
wait50:	clr	r17			;Zeitkonstante uere Schleife
	clr	r16			;Zeitkonstante innere Schleife
wait00:	dec	r16			;Zhler innere Schleife
	brne	wait00
	wdr				;Watchdog zurcksetzen
	dec	r17			;Zhler uere Schleife
	brne	wait00
	ret
;
;	------------------------------------------------
;	Erhhen des DCF77-Pufferinhaltes um eine Minute,
;	es werden nur Minuten und Stunden bercksichtigt
;	Register: r22,r23
;		  z   <- Zeiger auf DCF77-Puffer
;	------------------------------------------------
;
incdcb:	ldd	r23,z+1			;Minuten Einer holen
	inc	r23			;erhhen
	std	z+1,r23			;und wieder speichern
	cpi	r23,10			;bertrag?
	brcs	incd20			;nein -> weiter
	clr	r23			;sonst zurcksetzen
	std	z+1,r23			;und speichern
	ldd	r23,z+2			;Minuten Zehner holen
	inc	r23			;erhhen
	std	z+2,r23			;und wieder speichern
	cpi	r23,6			;bertrag?
	brcs	incd20			;nein -> weiter
	clr	r23			;sonst zurcksetzen
	std	z+2,r23			;und speichern
	ldd	r22,z+4			;Stunden Zehner holen
	ldd	r23,z+3			;Stunden Einer holen
	inc	r23			;erhhen
	std	z+3,r23			;und wieder speichern
	cpi	r23,10			;bertrag?
	brcs	incd10			;nein -> Test auf "24"
	clr	r23			;sonst zurcksetzen
	std	z+3,r23			;und speichern
	inc	r22			;Stunden Zehner erhhen
	std	z+4,r22			;und speichern
incd10:	cpi	r22,2			;Stunden Zehner=2?
	brne	incd20			;nein -> weiter
	cpi	r23,4			;Stunden Einer=4 (=24) ?
	brne	incd20			;nein -> weiter
	clr	r23			;sonst Stunden zurcksetzen
	std	z+3,r23			;Stunden Einer speichern
	std	z+4,r23			;Stunden Zehner speichern
incd20:	ret
;
;	-------------------------------------------------------------
;	Multiplikation 16*16 Bit ohne Vorzeichen (aus AppNote AVR200)
;	Register: r16,r17,r18,r19,r20,r21,r22
;		  r16 - Faktor 1 Low
;		  r17 - Faktor 1 High
;		  r18 - Faktor 2 Low  -> Produkt 0 (LSB)
;		  r19 - Faktor 2 High -> Produkt 1
;		  r20 -			 Produkt 2
;		  r21 -			 Produkt 3 (MSB)
;	-------------------------------------------------------------
;
mul16u:	clr	r21		;die 2 hchstwertigen Bytes lschen
	clr	r20
	ldi	r22,16		;Schleifenzhler initialisieren
	lsr	r19
	ror	r18
m16u10:	brcc	m16u20		;wenn Bit 0 in Faktor 2 gesetzt, dann
	add	r20,r16		;Faktor 1 mit Ergebnisbyte 2 addieren
	adc	r21,r17		;Faktor 1 mit Ergebnisbyte 3 addieren
m16u20:	ror	r21		;Ergebnisbyte 3 nach rechts schieben
	ror	r20		;Ergebnisbyte 2 nach rechts schieben
	ror	r19		;Ergebnisbyte 1 nach rechts schieben
	ror	r18		;Ergebnisbyte 0 nach rechts schieben
	dec	r22		;Schleifenzhler dekrementieren
	brne	m16u10		;weiter wenn Zhler>0
	ret

;	-------------------------------------------------------
;	Division 16/16 Bit ohne Vorzeichen (aus AppNote AVR200)
;	Register: r8,r9,r16,r17,r18,r19,r20
;		  r16 - Divident Low  -> Quotient Low
;		  r17 - Divident High -> Quotient High
;		  r18 - Divisor Low
;		  r19 - Divisor High
;		  r8  - Rest Low
;		  r9  - Rest High
;	-------------------------------------------------------
;
div16u:	clr	r8		;Rest L lschen
	sub	r9,r9		;Rest H und Carry lschen
	ldi	r20,17		;Schleifenzhler initialisieren
d16u10:	rol	r16		;Dividend nach links schieben
	rol	r17
	dec	r20		;Schleifenzhler decrementieren
	brne	d16u20		;weiter wenn Zhler>0
	ret			;sonst Ende
d16u20:	rol	r8		;Dividend in Rest schieben
	rol	r9
	sub	r8,r18		;Rest = Rest - Divisor
	sbc	r9,r19
	brcc	d16u30		;weiter wenn Ergebnis negativ
	add	r8,r18		;sonst Rest wieder holen
	adc	r9,r19
	clc			;Carry lschen und ins
	rjmp	d16u10		;Ergebnis schieben
d16u30:	sec			;oder Carry setzen und ins
	rjmp	d16u10		;Ergebnis schieben
;
;	-----------------------------------------------
;	Konvertierung Binr (10 Bit) -> BCD (3 Stellen)
;	Register r0,r1,r2,r16,r17
;	Eingang: r16 - Binrzahl low
;		 r17 - Binrzahl High
;	Ausgang: r0  - BCD-Ziffer Low
;		 r1  - BCD-Ziffer Mid
;		 r2  - BCD-Ziffer High
;	-----------------------------------------------
;
binbcd:	clr	r0			;0-Wert fr 16-Bit Subtraktion
	clr	r2			;BCD-High lschen
binb10:	mov	r1,r16			;Binrzahl zwischenspeichern L
	subi	r16,100			;100 subtrahieren
	sbc	r17,r0			;bertrag subtrahieren
	brcs	binb20			;war Zahl>99? nein -> weiter
	inc	r2			;BCD-High erhhen
	rjmp	binb10			;Schleife
binb20:	mov	r16,r1			;letzten Binrwert L holen
	clr	r1			;BCD-Mid lschen
binb30:	cpi	r16,10			;Wert>9?
	brcs	binb40			;nein -> Rest auswerten
	inc	r1			;sonst BCD-Mid erhhen
	subi	r16,10			;Wert um 10 vermindern
	rjmp	binb30			;Schleife
binb40:	mov	r0,r16			;Rest in BCD-Low legen
	ret
;
;	--------------------------------
;	Konvertierung BCD -> Hexadezimal
;	Register: r0  - BCD-Zahl L (0-9)
;		  r1  - BCD-Zahl H (0-9)
;		  r16 - Hexzahl (0-99)
;		  r17 - intern
;	--------------------------------
;
bcdhex:	clr	r16			;Ergebnis=0
	ldi	r17,10
bcdh10:	tst	r1			;H-Wert=0?
	breq	bcdh20			;ja -> L-Wert bearbeiten
	add	r16,r17			;sonst H-Wert +10
	dec	r1
	rjmp	bcdh10			;Schleife
bcdh20:	add	r16,r0			;L-Wert addieren
	ret
;
;	--------------------------------
;	Konvertierung Hexadezimal -> BCD
;	Register: r16 - Hexzahl (0-99)
;		  r0  - BCD-Zahl L (0-9)
;		  r1  - BCD-Zahl H (0-9)
;	--------------------------------
;
hexbcd:	clr	r1			;High-Zahl lschen
hexb10:	cpi	r16,10			;Hexwert>9?
	brcs	hexb20			;nein -> Low-Zahl bearbeiten
	inc	r1			;sonst High-Zahl erhhen
	subi	r16,10			;Hexwert-10
	rjmp	hexb10			;Schleife
hexb20:	mov	r0,r16			;Low-Zahl speichern
	ret
;
;	----------------------------------------------
;	Konvertierung BCD -> Siebensegment (Standard)
;	Register: r16 - Ziffer (0-F) -> 7-Segment-Code
;		  r0,r1,z - intern
;	----------------------------------------------
;
bcd7s1:	ldi	zl,low(segtb1*2)	;Zeiger auf Segment-Tabelle 1
	ldi	zh,high(segtb1*2)
	cpi	r16,16			;BCD-Wert<16?
	brcs	bcd700			;ja -> konvertieren
	ldi	r16,15			;sonst BCD-Wert=15 setzen
	rjmp	bcd700			;und konvertieren
;
;	-----------------------------------------------
;	Konvertierung BCD -> Siebensegment (Std-Zehner)
;	Register: r16 - Ziffer (0-3) -> 7-Segment-Code
;		  r0,r1,z - intern
;	-----------------------------------------------
;
bcd7s2:	ldi	zl,low(segtb2*2)	;Zeiger auf Segment-Tabelle 2
	ldi	zh,high(segtb2*2)
	cpi	r16,4			;BCD-Wert<4?
	brcs	bcd700			;ja -> konvertieren
	ldi	r16,3			;sonst BCD-Wert=3 setzen
bcd700:	clr	r1			;Wert fr bertragsberechnung
	add	zl,r16			;Tabellenoffset berechnen
	adc	zh,r1			;eventuell bertrag addieren
	lpm				;Segment-Code holen
	mov	r16,r0			;und ablegen
	ret
;
;	-----------------------------------------
;	Unterprogramm zum Lesen vom EEPROM
;	Register: r16 - EEPROM-Adresse Low
;		  r17 - EEPROM-Adresse High
;		  r0  - EEPROM-Daten
;	-----------------------------------------
;
eeread:	sbic	eecr,eewe		;luft Schreibzyklus?
	rjmp	eeread			;ja - warten
	out 	eearh,r17		;EEPROM-Adresse High setzen
	out	eearl,r16		;EEPROM-Adresse Low setzen
	sbi	eecr,eere		;EEPROM lesen aktivieren
	in	r0,eedr			;Daten lesen
	ret
;
;	-----------------------------------------
;	Unterprogramm zum Schreiben in den EEPROM
;	Register: r16 - EEPROM-Adresse Low
;		  r17 - EEPROM-Adresse High
;		  r18 - EEPROM-Daten
;		  r1  - interner Zwischenspeicher
;	-----------------------------------------
;
eewrit:	sbic	eecr,eewe		;luft Schreibzyklus?
	rjmp	eewrit			;ja -> warten
	out	eearh,r17		;EEPROM-Adresse H setzen
	out	eearl,r16		;EEPROM-Adresse L setzen
	out	eedr,r18		;EEPROM-Datenbyte setzen
	sbi	eecr,eemwe		;Masterwrite-Bit setzen
	sbi	eecr,eewe		;Write-Bit setzen
	ret
;
;	-------------------------------------------------
;	Unterprogramm zum Aktivieren des 3Wire-Interface
;	zur Kommunikation mit dem Temperatursensor DS1620
;	Register: keine
;	-------------------------------------------------
;
w3actv:	cbi	portd,0			;3Wire-DQ auf Low setzen
	sbi	portd,1			;3Wire-Clk auf High setzen
	nop				;warten
	sbi	portc,7			;2Wire-Rst auf High setzen
	ret
;
;	--------------------------------------------------
;	Unterprogramm zum Deaktivieren des 3Wire-Interface
;	Register: keine
;	--------------------------------------------------
;
w3deac:	cbi	portc,7			;3Wire-Rst in Ruhezustand
	cbi	portd,1			;3Wire-Clk auf Low setzen
	cbi	portd,0			;3Wire-DQ auf Low setzen
	ret
;
;	---------------------------------------------
;	Unterprogramm zum Senden eines Kommandos ber
;	3Wire-Interface zum Temperatursensor DS1620
;	Register: r20,r21,r22
;		  r22 - zu sendendes Byte
;	---------------------------------------------
;
w3send:	ldi	r21,8			;8 Bits senden
	in	r20,portd		;aktuellen PortD-Status holen
w3se10:	andi	r20,0b11111100		;Clk auf Low setzen
	bst	r22,0			;LSB vom Datenbyte holen
	bld	r20,0			;und als Datenbit einsetzen
	out	portd,r20		;ausgeben
	ori	r20,0b00000010		;Clk auf High (Datenbernahme)
	ror	r22			;nchstes Bit vorbereiten
	dec	r21			;alle Bits gesendet?
	out	portd,r20		;ausgeben
	brne	w3se10			;Schleife
	ret
;
;	-------------------------------------------
;	Unterprogramm zum Empfangen von 8 Bits ber
;	3Wire-Interface vom Temperatursensor DS1620
;	Register: r20,r21,r22,r23
;		  r22 - empfangenes Byte
;	-------------------------------------------
;
w3rec8:	cbi	ddrd,0			;DQ auf Input umschalten
	in	r20,portd		;aktuellen PortD-Status holen
	ori	r20,0b00000011		;Clk auf High setzen, DQ Pullup
	out	portd,r20		;ausgeben
	ldi	r21,8			;8 Datenbits einlesen
w3re10:	cbi	portd,1			;Clk auf Low (Datenbernahme)
	nop				;warten
	in	r23,pind		;Datenbit einlesen
	sbi	portd,1			;Clk auf High
	ror	r23			;empfangenes Bit in Carry und
	ror	r22			;in Datenbyte einschieben
	dec	r21			;alle Bits eingelesen?
	brne	w3re10			;nein -> Schleife
	sbi	ddrd,0			;sonst DQ auf Output umschalten
	ret
;
;	-------------------------------------------
;	Unterprogramm zum Empfangen von 9 Bits ber
;	3Wire-Interface vom Temperatursensor DS1620
;	Register: r20,r21,r22,r23
;		  r22 - empfangenes Byte (L)
;		  r23 - empfangenes Byte (H)
;	-------------------------------------------
;
w3rec9:	cbi	ddrd,0			;DQ auf Input umschalten
	in	r20,portd		;aktuellen PortD-Status holen
	ori	r20,0b00000011		;Clk auf High setzen, DQ Pullup
	out	portd,r20		;ausgeben
	ldi	r21,8			;8 Datenbits einlesen
w3re20:	cbi	portd,1			;Clk auf Low (Datenbernahme)
	nop				;warten
	in	r23,pind		;Datenbit einlesen
	sbi	portd,1			;Clk auf High
	ror	r23			;empfangenes Bit in Carry und
	ror	r22			;in Datenbyte einschieben
	dec	r21			;alle Bits eingelesen?
	brne	w3re20			;nein -> Schleife
	cbi	portd,1			;Clk auf Low (Datenbernahme)
	nop				;warten
	in	r23,pind		;Datenbit einlesen
	sbi	portd,1			;Clk auf High
	andi	r23,0x01		;MSB (Bit 9) in r23
	sbi	ddrd,0			;DQ auf Output umschalten
	ret
;
;	------------------------------------------------------------
;	Interrupt-Routine Timer1 Compare A, alle 10ms
;	liest DCF77 und Taste ein, zhlt die Uhrzeit
;	Register: dcfimp,dcfcnt,keycnt,flags,inttmp,intreg,r25,xl,xh
;	------------------------------------------------------------
;
t1int:	in	intreg,sreg		;CPU-Flags sichern
	wdr				;Watchdog zurcksetzen
	in	r25,pina		;Taste und DCF77 einlesen
	bst	r25,5			;DCF77-Status holen
	bld	flags,0			;und in Flags speichern
	inc	keycnt			;Tastenzhler erhhen
	bst	r25,4			;Tastenzustand sichern
	ldi	r25,4			;Vergleichswert laden
	brts	t1i010			;Taste gedrckt? nein -> weiter
	sbrc	flags,6			;Tastenstatus gedrckt?
	clr	keycnt			;ja, Tastenzhler lschen
	cp	keycnt,r25		;Taste 4 Zyklen gedrckt?
	brne	t1i020			;nein -> weiter
	clr	keycnt			;sonst Tastenzhler lschen
	ldi	r25,0b01000000		;Tastenflag setzen
	or	flags,r25		;und in Flags einsetzen
	rjmp	t1i020
;
t1i010:	clr	inttmp			;Tasterzhler lschen
	sts	keytim,inttmp		;und speichern
	sbrs	flags,6			;Tastenstatus losgelassen?
	clr	keycnt			;ja, Tastenzhler lschen
	cp	keycnt,r25		;Taste 4 Zyklen gedrckt?
	brne	t1i020			;nein -> weiter
	clr	keycnt			;sonst Tastenzhler lschen
	ldi	r25,0b00111111		;Tasten/Bearbeitungs-Flag
	and	flags,r25		;lschen und in Flags einsetzen
;
t1i020:	inc	dcfcnt			;DCF77-Zhler erhhen
	bst	flags,0			;akuellen DCF77-Status prfen
	brts	t1i040			;ist DCF77 High? ja -> weiter
	bst	flags,1			;sonst vorh. DCF-Status prfen
	brtc	t1i060			;war DCF77 vorher High?
	ldi	r25,150			;nein -> weiter
	cp	dcfcnt,r25		;Zhler<150 (1,5s) ?
	brcs	t1i030			;nein -> weiter
	bld	flags,3			;sonst Rahmenende setzen
;
t1i030:	ldi	r25,90			;Vergleichswert laden
	cp	dcfcnt,r25		;Zhler<90 (900ms) ?
	brcs	t1i060			;ja -> ignorieren
	clr	dcfcnt			;sonst DCF77-Zhler lschen
	rjmp	t1i060			;Ende
t1i040:	bst	flags,1			;vorherigen DCF-Status prfen
	brts	t1i060			;war DCF77 vorher H? ja -> Ende
	ldi	r25,25			;Vergleichswert laden
	cp	dcfcnt,r25		;DCF-Zhler<25 (250ms)?
	brcc	t1i060			;nein -> ignorieren
	ldi	r25,5			;Vergleichswert laden
	cp	dcfcnt,r25		;DCF-Zhler<5 (50ms)?
	brcs	t1i060			;ja -> ignorieren
	inc	dcfimp			;sonst Impuls zhlen
	ldi	r25,15			;DCF-Zhler<15 (150ms)?
	cp	dcfcnt,r25		;Ergebnis in Carry
	ldi	r25,6			;6 Pufferbytes bearbeiten
	ldi	xl,low(dcftab+6)	;Zeiger auf Pufferende setzen
	ldi	xh,high(dcftab+6)
	rol	inttmp			;Carry in Register legen
	com	inttmp			;alle Bits invertieren
	ror	inttmp			;wieder in Carry schieben
t1i050:	ld	inttmp,-x		;Pufferbyte holen
	ror	inttmp			;neues Bit einschieben als MSB
	st	x,inttmp		;LSB in Carry und speichern
	dec	r25			;alle Pufferbytes bearbeitet?
	brne	t1i050			;nein -> Schleife
t1i060:	bst	flags,0			;aktuellen DCF77-Status holen
	bld	flags,1			;und als neuen Status speichern
;
	in	r25,adcl		;A/D-Wandler-Wert L lesen
	in	inttmp,adch		;A/D-Wandler-Wert H lesen
	ror	inttmp			;H-Wert halbieren
	ror	r25			;L-Wert halbieren
	ror	inttmp			;H-Wert halbieren
	ror	r25			;L-Wert halbieren
	sts	adcval,r25		;8-Bit-Ergebnis speichern
;
	lds	r25,pcount		;Vorteiler holen
	tst	r25			;Vorteiler=0?
	brne	t1i090			;nein -> berspringen
	lds	r25,second		;Sekunden holen
	cpi	r25,0			;Sekundenwert=0?
	breq	t1i080			;ja -> bearbeiten
	cpi	r25,10			;Sekundenwert=10?
	breq	t1i080			;ja -> bearbeiten
	cpi	r25,20			;Sekundenwert=20?
	breq	t1i080			;ja -> bearbeiten
	cpi	r25,30			;Sekundenwert=30?
	breq	t1i080			;ja -> bearbeiten
	cpi	r25,40			;Sekundenwert=40?
	breq	t1i080			;ja -> bearbeiten
	cpi	r25,50			;Sekundenwert=50?
	breq	t1i080			;ja -> bearbeiten
	rjmp	t1i090			;sonst berspringen
t1i080:	lds	inttmp,tmpflg		;Temperatur-Flags holen
	clt				;Temperatur-Flag
	bld	inttmp,0		;lschen
	sts	tmpflg,inttmp		;Flags wieder speichern
;
t1i090:	lds	r25,nightf		;Nachtflag holen
	tst	r25			;Nachtmodus aktiv?
	breq	t1i100			;nein -> berspringen
	lds	r25,pcount		;sonst Vorteiler holen
	lds	inttmp,second		;Sekunden holen
	or	r25,inttmp		;Werte verknpfen
	lds	inttmp,minute		;Minuten holen
	or	r25,inttmp		;sind alle Werte 0?
	brne	t1i100			;nein -> berspringen
	lds	inttmp,tmpflg		;sonst Temperatur-Flags holen
	set				;Temperaturmess-Flag
	bld	inttmp,1		;setzen und
	sts	tmpflg,inttmp		;Flags wieder speichern
;
t1i100:	lds	r25,pcount		;Vorteiler holen
	inc	r25			;erhhen
	sts	pcount,r25		;und wieder speichern
	cpi	r25,100			;Endwert erreicht?
	brne	t1i190			;nein -> Ende
	clr	r25			;sonst zurcksetzen
	sts	pcount,r25		;und speichern
;
	lds	r25,keytim		;Tasterzhler holen
	sbrc	flags,6			;Tastenstatus gedrckt?
	inc	r25			;ja, Tasterzhler erhhen
	sts	keytim,r25		;Tasterzhler wieder speichern
;
	lds	r25,ditime		;Timeout-Zhler holen
	tst	r25			;ist Zhler bereits auf 0?
	breq	t1i110			;ja -> Ende
	dec	r25			;sonst vermindern
	sts	ditime,r25		;und wieder speichern
;
t1i110:	lds	r25,second		;Sekunden holen
	inc	r25			;erhhen
	sts	second,r25		;und wieder speichern
;
	cpi	r25,60			;Endwert erreicht?
	brne	t1i190			;nein -> Ende
	clr	r25			;sonst zurcksetzen
	sts	second,r25		;und speichern
;
	lds	r25,minute		;Minuten holen
	inc	r25			;erhhen
	sts	minute,r25		;und wieder speichern
	cpi	r25,60			;Endwert erreicht?
	brne	t1i190			;nein -> Ende
	clr	r25			;sonst zurcksetzen
	sts	minute,r25		;und speichern
;
	lds	r25,hour		;Stunden holen
	inc	r25			;erhhen
	sts	hour,r25		;und wieder speichern
	cpi	r25,24			;Endwert erreicht?
	brne	t1i190			;nein -> Ende
	clr	r25			;sonst zurcksetzen
	sts	hour,r25		;und speichern
;
	lds	r25,day			;Tag holen
	inc	r25			;erhhen
	sts	day,r25			;und wieder speichern
	lds	xl,month		;Monat holen
	cpi	xl,1			;Monat Januar?
	brne	t1i200			;nein -> weiter testen
	cpi	r25,32			;Tag>31?
	brcc	t1i210			;ja -> Monatswechsel
t1i190:	rjmp	timend			;sonst Ende
;
t1i200:	cpi	xl,2			;Monat Februar?
	brne	t1i230			;nein -> weiter testen
	lds	xl,year			;Jahr holen
	andi	xl,0x03			;Schaltjahr?
	breq	t1i220			;ja -> 29 Tage
	cpi	r25,29			;sonst Tag>28?
t1i210:	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
t1i220:	cpi	r25,30			;Tag>29?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i230:	cpi	xl,3			;Monat Mrz?
	brne	t1i240			;nein -> weiter testen
	cpi	r25,32			;Tag>31?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i240:	cpi	xl,4			;Monat April?
	brne	t1i250			;nein -> weiter testen
	cpi	r25,31			;Tag>30?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i250:	cpi	xl,5			;Monat Mai?
	brne	t1i260			;nein -> weiter testen
	cpi	r25,32			;Tag>31?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i260:	cpi	xl,6			;Monat Juni?
	brne	t1i270			;nein -> weiter testen
	cpi	r25,31			;Tag>30?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i270:	cpi	xl,7			;Monat Juli?
	brne	t1i280			;nein -> weiter testen
	cpi	r25,32			;Tag>31?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i280:	cpi	xl,8			;Monat August?
	brne	t1i290			;nein -> weiter testen
	cpi	r25,32			;Tag>31?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i290:	cpi	xl,9			;Monat September?
	brne	t1i300			;nein -> weiter testen
	cpi	r25,31			;Tag>30?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i300:	cpi	xl,10			;Monat Oktober?
	brne	t1i310			;nein -> weiter testen
	cpi	r25,32			;Tag>31?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i310:	cpi	xl,11			;Monat November?
	brne	t1i320			;nein -> Monat Dezember
	cpi	r25,31			;Tag>30?
	brcc	t1i400			;ja -> Monatswechsel
	rjmp	timend			;sonst Ende
;
t1i320:	cpi	r25,32			;Tag>31?
	brcs	timend			;nein -> Ende
;
t1i400:	ldi	r25,1			;Tag=1 setzen
	sts	day,r25			;und speichern
	lds	r25,month		;Monat holen
	inc	r25			;erhhen
	sts	month,r25		;und wieder speichern
	cpi	r25,13			;Endwert berschritten?
	brcs	timend			;nein -> Ende
	ldi	r25,1			;sonst Monat=1 setzen
	sts	month,r25		;und wieder speichern
	lds	r25,year		;Jahr holen
	inc	r25			;erhhen
	sts	year,r25		;und wieder speichern
;
timend:	out	sreg,intreg		;CPU-Flags wiederherstellen
	reti				;Ende
;
segtb1:	.db	0b01000000,0b01111001	;Code 0/1: Zeichen "0,1"
	.db	0b00100100,0b00110000	;Code 2/3: Zeichen "2,3"
	.db	0b00011001,0b00010010	;Code 4/5: Zeichen "4,5"
	.db	0b00000010,0b01111000	;Code 6/7: Zeichen "6,7"
	.db	0b00000000,0b00010000	;Code 8/9: Zeichen "8,9"
	.db	0b00001100,0b00101111	;Code A/B: Zeichen "P,r"
	.db	0b00011100,0b00101100	;Code C/D: Zeichen ",?"
	.db	0b00111111,0b01111111	;Code E/F: Zeichen "-, "
;
segtb2:	.db	0b00001111,0b00001100	;Code 0/1: Zeichen " ,1"
	.db	0b00000010,0b00000100	;Code 2/3: Zeichen "2,3"
;
versnr:	.db	0,1,1,5			;Version 01.15
;
invfl1:	.db	0xff,0	;Flag fr die Invertierung der Ausgnge; nur
			;nur das erste Byte ist relevant
			;0= Normalbetrieb, 0xff= Inversbetrieb
;
spflg1:	.db	0xff,0	;Spezial-Flag fr die Doppelpunkt-Anzeige
			;nur das erste Byte ist relevant
			;0= Normalbetrieb, 0xff= Doppelpunktmodus
;
.eseg
;
dummy:	.db	0xff	;wird nicht genutzt
edprog:	.db	0	;gespeichertes Anzeigeprogramm
			;wird auf Anzeige als 1,2,3 dargestellt:
			;0: Anzeige Zeit, Datum, Temperatur
			;1: Anzeige Zeit, Temperatur
			;2: Anzeige Zeit
edimod:	.db	0	;zuletzt aktiver Anzeigemodus
			;0: normale Zeitanzeige
			;1: Sekundenanzeige
			;2: Temperatur-Anzeige
;
